/*
 * Copyright (C) 2006 Johan Maasing johan at zoom.nu Licensed under the Apache
 * License, Version 2.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law
 * or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package nu.zoom.catonine.desktop.impl;

import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComponent;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.KeyStroke;

import nu.zoom.catonine.desktop.ImportStyleRulesManagerAction;
import nu.zoom.catonine.desktop.OpenStyleRulesManagerAction;
import nu.zoom.catonine.desktop.SaveStyleRulesManagerAsAction;
import nu.zoom.catonine.desktop.StyleRulesManagerComponent;
import nu.zoom.catonine.desktop.StyleRulesPlugIn;
import nu.zoom.catonine.prefs.Preferences;
import nu.zoom.catonine.stylerule.SampleStyleRulesFactory;
import nu.zoom.catonine.stylerule.StyleRulesManager;
import nu.zoom.catonine.stylerule.StyleRulesManagerListener;
import nu.zoom.catonine.xml.StyleRulesManagerPersistence;
import nu.zoom.catonine.xml.StyleRulesManagerPersistence.FileCanNotBeReadException;
import nu.zoom.gal.error.ErrorReporter;
import nu.zoom.swing.desktop.CloseVetoListener;
import nu.zoom.swing.desktop.Workbench;
import nu.zoom.swing.desktop.WorkbenchFrame;
import nu.zoom.swing.desktop.WorkbenchListener;
import nu.zoom.swing.desktop.WorkbenchMenuBar;
import nu.zoom.swing.desktop.common.BackendException;
import nu.zoom.swing.desktop.common.action.ToggleFrameAction;
import nu.zoom.swing.desktop.component.filechooser.FileChooser;
import nu.zoom.swing.desktop.preferences.InvalidDataTypeException;
import nu.zoom.ui.Resources;
import nu.zoom.ui.Resources.ResourceNotFoundException;

/**
 * A plug-in for the zoom.nu desktop to manage collections of style rules.
 * 
 * @author "Johan Maasing" &lt;johan@zoom.nu&gt;
 * 
 */
public class StyleRulesPlugInImpl implements StyleRulesPlugIn, WorkbenchListener, CloseVetoListener {

	private StyleRulesManager manager;
	private final Workbench workbench;
	private final Resources messages;
	private final Preferences preferences;
	private final ErrorReporter errorReporter;
	final ArrayList<StyleRulesManagerListener> listeners = new ArrayList<StyleRulesManagerListener>();
	StyleRulesManagerComponent styleRulesManagerComponent;

	public StyleRulesPlugInImpl(final Workbench workbench, final Resources messages, final Preferences preferences,
			final ErrorReporter errorReporter) {
		super();
		this.workbench = workbench;
		this.messages = messages;
		this.preferences = preferences;
		this.errorReporter = errorReporter;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see nu.zoom.swing.desktop.PlugIn#initialize()
	 */
	@Override
	public void initialize() throws Exception {
		workbench.addWorkBenchListener(this);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see nu.zoom.swing.desktop.WorkbenchListener#start()
	 */
	@Override
	public void start() {
		final ShowStyleRulesAction showStyleRulesAction;
		try {
			showStyleRulesAction = new ShowStyleRulesAction(messages.getMessage("nu.zoom.catonine.configurations"));
			final WorkbenchMenuBar menuBar = workbench.getMenuBar();
			menuBar.addToApplicationMenu(showStyleRulesAction.getControl());
			menuBar.addToApplicationMenu(new JMenuItem(new SaveStyleRulesManagerAsAction(this, messages)));
			menuBar.addToApplicationMenu(new JMenuItem(new OpenStyleRulesManagerAction(this, messages)));
			menuBar.addToApplicationMenu(new JMenuItem(new ImportStyleRulesManagerAction(this, messages, errorReporter)));
			workbench.addCloseVetoListener(this);
			workbench.registerKeyboardAction(showStyleRulesAction,
					KeyStroke.getKeyStroke(KeyEvent.VK_N, InputEvent.CTRL_DOWN_MASK),
					JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
			try {
				if (!loadManager()) {
					manager = new StyleRulesManager();
					manager.add(SampleStyleRulesFactory.getLog4JStyleRules());
					manager.add(SampleStyleRulesFactory.getSyslogStyleRules());
				}
			} catch (InvalidDataTypeException invalidDataTypeException) {
				errorReporter.reportError(invalidDataTypeException);
			} catch (BackendException backendException) {
				errorReporter.reportError(backendException);
			} catch (IOException e) {
				errorReporter.reportError(e);
			}
		} catch (Resources.ResourceNotFoundException resourceNotFoundException) {
			errorReporter.reportError(resourceNotFoundException);
		}
	}

	@Override
	public void close() {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see nu.zoom.swing.desktop.CloseVetoListener#canClose()
	 */
	@Override
	public boolean canClose() {
		if (manager.isChanged()) {
			int result;
			try {
				result =
						JOptionPane
								.showConfirmDialog(workbench.getDialogOwner(),
										messages.getMessage("nu.zoom.catonine.unsaved"),
										messages.getMessage("nu.zoom.catonine.unsaved.title"),
										JOptionPane.YES_NO_CANCEL_OPTION);
				switch (result) {
					case JOptionPane.YES_OPTION:
						saveManager(false);
						return true;
					case JOptionPane.NO_OPTION:
						return true;
					default:
						return false;
				}
			} catch (ResourceNotFoundException ex) {
				Logger.getLogger(StyleRulesPlugInImpl.class.getName()).log(Level.SEVERE, null, ex);
			}
		}
		return true;
	}

	public class ShowStyleRulesAction extends ToggleFrameAction<JCheckBoxMenuItem> {

		private static final long serialVersionUID = 1L;

		public ShowStyleRulesAction(final String name) {
			super(new JCheckBoxMenuItem());
			setName(name);
			setMnemonicKey(KeyEvent.VK_N);
			setAcceleratorKey(KeyEvent.VK_N, KeyEvent.CTRL_DOWN_MASK);
		}

		@Override
		protected WorkbenchFrame createFrame() {
			try {

				final StyleRulesManagerComponent newStyleRulesManagerComponent =
						new StyleRulesManagerComponent(manager, messages, workbench);
				if (StyleRulesPlugInImpl.this.styleRulesManagerComponent != null) {
					listeners.remove(StyleRulesPlugInImpl.this.styleRulesManagerComponent);
				}
				listeners.add(newStyleRulesManagerComponent);
				StyleRulesPlugInImpl.this.styleRulesManagerComponent = newStyleRulesManagerComponent;
				String title = null;
				try {
					title = preferences.getConfigurationFilename();
					if (title != null) {
						File titleFile = new File(title);
						title = titleFile.getName();
					}
				} catch (InvalidDataTypeException ex) {
					// TODO: Log error. But ignore since we will use default for
					// title
				} catch (BackendException ex) {
					// TODO: Log error. But ignore since we will use default for
					// title
				}
				if (title == null) {
					title = messages.getMessage("nu.zoom.catonine.configurations");
				}
				WorkbenchFrame managerFrame =
						workbench.createWorkbenchFrame("StyleRulesColoring", newStyleRulesManagerComponent, null, true,
								true);
				managerFrame.setTitle(title);
				return managerFrame;
			} catch (ResourceNotFoundException ex) {
				errorReporter.reportError(ex);
				return null;
			}
		}

		@Override
		public void frameDetached(WorkbenchFrame frame) {
		}

		@Override
		public void frameAttached(WorkbenchFrame frame) {
		}
	}

	private boolean loadManager() throws InvalidDataTypeException, BackendException, ResourceNotFoundException,
			IOException {
		boolean success = loadManager(preferences.getConfigurationFilename());
		if (!success) {
			// Failed to load default manager. Remove the preferences string
			// since it is obviously wrong.
			preferences.setConfigurationFilename(null);
		}
		return success;
	}

	public void importManager() throws ResourceNotFoundException, IOException, InvalidDataTypeException,
			BackendException {
		if (this.manager == null) {
			openManager();
		} else {
			final FileChooser fileChooser = preferences.getConfiguratonFilechooser();
			File openFile = fileChooser.openFile();
			if (openFile != null) {
				StyleRulesManager importFrom = readManager(openFile);
				if (importFrom != null) {
					// workbench.startWorkIndicator();
					this.manager.importManager(importFrom);
					workbench.getStatusbar().setMessage(
							messages.format("nu.zoom.catonine.import.from", openFile.getPath()));
					// workbench.stopWorkIndicator();
				}
			}
		}
	}

	public boolean openManager() {
		try {
			final FileChooser fileChooser = preferences.getConfiguratonFilechooser();
			File openFile = fileChooser.openFile();
			if (openFile != null) {
				try {
					final boolean success = loadManager(openFile.getCanonicalPath());
					if (success) {
						fireStyleRulesManagerChanged();
					}
					return success;
				} catch (IOException ex) {
					String message = messages.getMessage("nu.zoom.catonine.configurations.load.failed");
					errorReporter.reportError(message, ex);
				}
			}
		} catch (ResourceNotFoundException ex1) {
			errorReporter.reportError("Unable to load resource", ex1);
		} catch (InvalidDataTypeException e) {
			errorReporter.reportError("Unable to retrieve filechooser preferences", e);
		} catch (BackendException e) {
			errorReporter.reportError("Unable to retrieve filechooser preferences", e);
		}
		return false;
	}

	private boolean loadManager(final String managerFileName) throws ResourceNotFoundException, IOException {
		if (managerFileName == null) {
			return false;
		}
		if ((manager != null) && (manager.isChanged())) {
			saveManager(false);
		}
		StyleRulesManager newManager = readManager(managerFileName);
		// If we load a new manager when an old is opened we should inform
		// listeners (GUI elements) that the rules have changed.
		if (newManager != null) {
			if (manager != null) {
				manager.copyListenersTo(newManager);
			}
			manager = newManager;
			try {
				preferences.setConfigurationFilename(managerFileName);
			} catch (BackendException ex) {
				// TODO: log error but we ignore it. If not set it only affects
				// the frame title.
			} catch (InvalidDataTypeException e) {
				// TODO: log error but we ignore it. If not set it only affects
				// the frame title.
			}
			return true;
		}
		return false;
	}

	private StyleRulesManager readManager(String managerFileName) throws ResourceNotFoundException, IOException {
		StyleRulesManager newManager = null;
		if (managerFileName != null) {
			final File managerFile = new File(managerFileName);
			newManager = readManager(managerFile);
		}
		return newManager;
	}

	private StyleRulesManager readManager(File managerFile) throws ResourceNotFoundException, IOException {
		StyleRulesManager newManager = null;
		try {
			newManager = StyleRulesManagerPersistence.readManager(managerFile);
		} catch (FileNotFoundException exc) {
			final String message =
					messages.format("nu.zoom.catonine.open.fail.noexist", managerFile.getCanonicalPath());
			errorReporter.reportError(message);
		} catch (FileCanNotBeReadException exc) {
			final String message = messages.format("nu.zoom.catonine.open.fail.noread", managerFile.getCanonicalPath());
			errorReporter.reportError(message);

		} catch (Exception e) {
			errorReporter.reportError(messages.getMessage("nu.zoom.catonine.configurations.load.failed"), e);
			newManager = null;
		}
		return newManager;
	}

	public void saveManager(final boolean saveAs) {
		try {
			String managerFileName = preferences.getConfigurationFilename();
			// there is no default name or we should select a new name
			if ((managerFileName == null) || saveAs) {
				FileChooser fileChooser = preferences.getConfiguratonFilechooser();
				File selectedSaveFile = fileChooser.saveFile();
				if (selectedSaveFile != null) {
					managerFileName = selectedSaveFile.getCanonicalPath();
				}
			}
			// make sure it ends with .xml
			if (managerFileName != null) {
				if (!managerFileName.endsWith(".xml")) {
					managerFileName = managerFileName + ".xml";
				}
				final File saveFile = new File(managerFileName);
				StyleRulesManagerPersistence.save(manager, saveFile);
				manager.resetChangeStatus();
				preferences.setConfigurationFilename(saveFile.getCanonicalPath());
			}
		} catch (Exception e) {
			try {
				errorReporter.reportError(messages.getMessage("nu.zoom.catonine.configurations.save.failed"), e);
			} catch (ResourceNotFoundException ex) {
				errorReporter.reportError(ex);
			}
		}
	}

	private void fireStyleRulesManagerChanged() {
		for (StyleRulesManagerListener styleRulesManagerListener : listeners) {
			styleRulesManagerListener.styleRulesChanged(manager);
		}
	}

	@Override
	public StyleRulesManager getStyleRulesManager() {
		return manager;
	}
}
